/*****************************************************************************
  File:		serial.c                                              
  Purpose:		Board specific routines for the MPC5675K              
 ****************************************************************************/
#include "mpc5675k-2.04.h"
//#include "mpc5675k_0201.h"
#include "serial.h"
#include "common.h"

extern uint32_t SYS_CLK;

// SCI_A ***********************************************************************
// --------------------------------------------------------------------------
//	Initialise serial port SCI_A
//	Use Baud 57600kbps / No par / 8 data bits / 1 stop bit

void sci_a_init(){
    
  // Put the LINFlex into Init Mode 
  LINFLEX_0.LINCR1.B.INIT = 1;
  LINFLEX_0.LINCR1.B.SLEEP = 0;
  
  // Enable UART Mode 
  LINFLEX_0.UARTCR.B.UART = 1;
  
  // Set the baud rate
  LINFLEX_0.LINIBRR.R = BAUD_INT;
  LINFLEX_0.LINFBRR.R = BAUD_FRAC;
    
  // No parity, 1 stop bit, 1 start bit, No loopback   
  // 8-bits per character, idle bit count after stop bit 
  LINFLEX_0.UARTCR.B.WL0 = 1;
                 
  // Enable receiver and transmitter 
  LINFLEX_0.UARTCR.B.RXEN = 1;
  LINFLEX_0.UARTCR.B.TXEN = 1;
  
 // Configure the LINFLEX Output Pins   
  SIU.PCR[18].R = 0x0600;	// Set the LIN0_TX pad for operation    
  SIU.PCR[19].R = 0x0100;	// Enable Input Buffer PSMI default is ok  

  // Exit Init Mode 
  LINFLEX_0.LINCR1.B.INIT = 0;
  
}
// --------------------------------------------------------------------------
//	receive character form SCI_A
char sci_a_getchar (void){

	// Wait until character has been received 
	while (LINFLEX_0.UARTSR.B.DRFRFE != 1){;}

	// Check Buffer can be read 
	while (LINFLEX_0.UARTSR.B.RMB != 1){;}

	LINFLEX_0.UARTSR.B.DRFRFE = 1;  // clear RX full flag 
	LINFLEX_0.UARTSR.B.RMB = 1;     // Release message buffer 

	return (char) LINFLEX_0.BDRM.B.DATA4;
}
// --------------------------------------------------------------------------
//	send character to SCI_A
void sci_a_putchar (char ch){

	LINFLEX_0.BDRL.B.DATA0 = (uint8_t)ch;	// Send the character 
	while (LINFLEX_0.UARTSR.B.DTFTFF != 1){;}	// wait for TX to complete 

	LINFLEX_0.UARTSR.B.DTFTFF = 1;	// Clear the TX complete flag 
	SIU.GPDO[18].B.PDO = 1;
}

// --------------------------------------------------------------------------
//	send character to SCI_A - if \n sends \r also
void sci_a_putch (char ch){

	sci_a_putchar(ch);
	if(ch == '\n'){
		sci_a_putchar('\r');
	}
}

// --------------------------------------------------------------------------
//	send string to SCI_A
void sci_a_puts (char *str){
	
	while(*str){	
		sci_a_putch(*str++);
	}
}
// --------------------------------------------------------------------------
// return receive status of SCI_A
int sci_a_getchar_present (void){
    return LINFLEX_0.UARTSR.B.DRFRFE;
}


// SCI_D ***********************************************************************

#pragma push
#pragma section code_type ".text_p1"
#pragma force_active on

// --------------------------------------------------------------------------
//	Initialise serial port SCI_D
//	Use Baud 57600kbps / No par / 8 data bits / 1 stop bit

void sci_d_init(){
    
  // Put the LINFlex into Init Mode 
  LINFLEX_3.LINCR1.B.INIT = 1;
  LINFLEX_3.LINCR1.B.SLEEP = 0;
  
  // Enable UART Mode 
  LINFLEX_3.UARTCR.B.UART = 1;
  
  // Set the baud rate
  LINFLEX_3.LINIBRR.R = BAUD_INT;
  LINFLEX_3.LINFBRR.R = BAUD_FRAC;
   
  // No parity, 1 stop bit, 1 start bit, No loopback   
  // 8-bits per character, idle bit count after stop bit 
  LINFLEX_3.UARTCR.B.WL0 = 1;
                 
  // Enable receiver and transmitter 
  LINFLEX_3.UARTCR.B.RXEN = 1;
  LINFLEX_3.UARTCR.B.TXEN = 1;
  
 // Configure the LINFLEX Output Pins   
  SIU.PCR[131].R = 0x0A00;	// Set the LIN3_TX pad for operation    
	SIU.PSMI[50].B.PADSEL = 0x02;	// this LIN3_RX is muxed, assign it to GPIO[143] 
  SIU.PCR[143].R = 0x0100;	// Enable Input Buffer PSMI default is ok  

  // Exit Init Mode 
  LINFLEX_3.LINCR1.B.INIT = 0;
  
}
// --------------------------------------------------------------------------
//	receive character form SCI_D
char sci_d_getchar (void){

	// Wait until character has been received 
	while (LINFLEX_3.UARTSR.B.DRFRFE != 1){;}

	// Check Buffer can be read 
	while (LINFLEX_3.UARTSR.B.RMB != 1){;}

	LINFLEX_3.UARTSR.B.DRFRFE = 1;  // clear RX full flag 
	LINFLEX_3.UARTSR.B.RMB = 1;     // Release message buffer 

	return (char) LINFLEX_3.BDRM.B.DATA4;
}
// --------------------------------------------------------------------------
//	send character to SCI_D
void sci_d_putchar (char ch){

	LINFLEX_3.BDRL.B.DATA0 = (uint8_t)ch;	// Send the character 

	// wait for TX to complete 
	while (LINFLEX_3.UARTSR.B.DTFTFF != 1){;}

	LINFLEX_3.UARTSR.B.DTFTFF = 1;	// Clear the TX complete flag 
	SIU.GPDO[131].B.PDO = 1;
}
// --------------------------------------------------------------------------
//	send character to SCI_D - if \n sends \r also
void sci_d_putch (char ch){

	sci_d_putchar(ch);
	if(ch == '\n'){
		sci_d_putchar('\r');
	}
}
// --------------------------------------------------------------------------
//	send string to SCI_D
void sci_d_puts (char *str){
	
	while(*str){	
		sci_d_putch(*str++);
	}
}

// --------------------------------------------------------------------------
// return receive status of SCI_D
int sci_d_getchar_present (void){
    return LINFLEX_3.UARTSR.B.DRFRFE;
}

#pragma pop
